@@ -1,6 +1,5 @@
-use std::collections::{HashMap, HashSet};
+use std::collections::HashMap;
 use std::fmt::Debug;
-use std::path::PathBuf;
 
 use protocol::perform_handshake;
 use stream::WsStream;
@@ -13,8 +12,11 @@ pub mod errors;
 pub mod frame;
 /// build connection & read/write frame utils
 pub mod protocol;
+
+#[cfg(feature = "proxy")]
 /// connection proxy support
 pub mod proxy;
+
 /// stream definition
 pub mod stream;
 
@@ -24,8 +26,8 @@ pub mod codec;
 use errors::WsError;
 use tokio_util::codec::{Decoder, Encoder, Framed};
 
+use crate::protocol::handle_handshake;
 use crate::protocol::Mode;
-use crate::protocol::{handle_handshake, wrap_tls};
 
 /// websocket connection state
 #[derive(Debug, Clone, PartialEq, Eq)]
@@ -44,10 +46,12 @@ pub enum ConnectionState {
 
 pub struct ClientBuilder {
     uri: String,
+    #[cfg(feature = "proxy")]
     proxy_uri: Option<String>,
-    protocols: HashSet<String>,
-    extensions: HashSet<String>,
-    certs: HashSet<PathBuf>,
+    protocols: Vec<String>,
+    extensions: Vec<String>,
+    #[cfg(feature = "rustls")]
+    certs: std::collections::HashSet<std::path::PathBuf>,
     version: u8,
     headers: HashMap<String, String>,
 }
@@ -56,15 +60,18 @@ impl ClientBuilder {
     pub fn new<S: ToString>(uri: S) -> Self {
         Self {
             uri: uri.to_string(),
+            #[cfg(feature = "proxy")]
             proxy_uri: None,
-            protocols: HashSet::new(),
-            extensions: HashSet::new(),
+            protocols: vec![],
+            extensions: vec![],
             headers: HashMap::new(),
-            certs: HashSet::new(),
+            #[cfg(feature = "rustls")]
+            certs: std::collections::HashSet::new(),
             version: 13,
         }
     }
 
+    #[cfg(feature = "proxy")]
     pub fn proxy<S: ToString>(self, uri: S) -> Self {
         Self {
             proxy_uri: Some(uri.to_string()),
@@ -74,39 +81,41 @@ impl ClientBuilder {
 
     /// add protocols
     pub fn protocol(mut self, protocol: String) -> Self {
-        self.protocols.insert(protocol);
+        self.protocols.push(protocol);
         self
     }
 
     /// set extension in handshake http header
     ///
     /// **NOTE** it will clear protocols set by `protocol` method
-    pub fn protocols(self, protocols: HashSet<String>) -> Self {
+    pub fn protocols(self, protocols: Vec<String>) -> Self {
         Self { protocols, ..self }
     }
 
     /// add protocols
     pub fn extension(mut self, extension: String) -> Self {
-        self.extensions.insert(extension);
+        self.extensions.push(extension);
         self
     }
 
     /// set extension in handshake http header
     ///
     /// **NOTE** it will clear protocols set by `protocol` method
-    pub fn extensions(self, extensions: HashSet<String>) -> Self {
+    pub fn extensions(self, extensions: Vec<String>) -> Self {
         Self { extensions, ..self }
     }
 
-    pub fn cert(mut self, cert: PathBuf) -> Self {
+    #[cfg(feature = "rustls")]
+    pub fn cert(mut self, cert: std::path::PathBuf) -> Self {
         self.certs.insert(cert);
         self
     }
 
+    #[cfg(feature = "rustls")]
     // set ssl certs in wss connection
     ///
     /// **NOTE** it will clear certs set by `cert` method
-    pub fn certs(self, certs: HashSet<PathBuf>) -> Self {
+    pub fn certs(self, certs: std::collections::HashSet<std::path::PathBuf>) -> Self {
         Self { certs, ..self }
     }
 
@@ -127,9 +136,11 @@ impl ClientBuilder {
     async fn _connect(&self) -> Result<(String, http::Response<()>, WsStream), WsError> {
         let Self {
             uri,
+            #[cfg(feature = "proxy")]
             proxy_uri,
             protocols,
             extensions,
+            #[cfg(feature = "rustls")]
             certs,
             version,
             headers,
@@ -146,14 +157,10 @@ impl ClientBuilder {
         } else {
             Err(WsError::InvalidUri("missing ws or wss schema".to_string()))
         }?;
+        #[cfg(feature = "rustls")]
         if mode == Mode::WS && !certs.is_empty() {
             tracing::warn!("setting tls cert has no effect on insecure ws")
         }
-        let ws_proxy: Option<proxy::Proxy> = match proxy_uri {
-            Some(uri) => Some(uri.parse()?),
-            None => None,
-        };
-
         let host = uri
             .host()
             .ok_or_else(|| WsError::InvalidUri(format!("can not find host {}", self.uri)))?;
@@ -162,33 +169,63 @@ impl ClientBuilder {
             None => mode.default_port(),
         };
 
-        let stream = match &ws_proxy {
-            Some(proxy_conf) => proxy_conf.connect((host, port)).await?,
-            None => TcpStream::connect((host, port)).await.map_err(|e| {
+        let stream;
+        #[cfg(feature = "proxy")]
+        {
+            let ws_proxy: Option<proxy::Proxy> = match proxy_uri {
+                Some(uri) => Some(uri.parse()?),
+                None => None,
+            };
+            stream = match &ws_proxy {
+                Some(proxy_conf) => proxy_conf.connect((host, port)).await?,
+                None => TcpStream::connect((host, port)).await.map_err(|e| {
+                    WsError::ConnectionFailed(format!(
+                        "failed to create tcp connection {}",
+                        e.to_string()
+                    ))
+                })?,
+            };
+        }
+
+        #[cfg(not(feature = "proxy"))]
+        {
+            stream = TcpStream::connect((host, port)).await.map_err(|e| {
                 WsError::ConnectionFailed(format!(
                     "failed to create tcp connection {}",
                     e.to_string()
                 ))
-            })?,
-        };
+            })?;
+        }
+
         tracing::debug!("tcp connection established");
+
         let mut stream = match mode {
             Mode::WS => WsStream::Plain(stream),
             Mode::WSS => {
-                let tls_stream = wrap_tls(stream, host, &self.certs).await?;
-                WsStream::Tls(tls_stream)
+                #[cfg(feature = "rustls")]
+                {
+                    use crate::protocol::wrap_tls;
+
+                    let tls_stream = wrap_tls(stream, host, &self.certs).await?;
+                    WsStream::Tls(tls_stream)
+                }
+
+                #[cfg(not(feature = "rustls"))]
+                {
+                    panic!("require `rustls`")
+                }
             }
         };
         let (key, resp) = perform_handshake(
             &mut stream,
             &mode,
             &uri,
-            protocols.iter().cloned().collect::<Vec<String>>().join(" "),
+            protocols.iter().cloned().collect::<Vec<String>>().join(" ,"),
             extensions
                 .iter()
                 .cloned()
                 .collect::<Vec<String>>()
-                .join(" "),
+                .join(" ,"),
             *version,
             headers.clone(),
         )
